@@ -92,7 +92,7 @@ def RunCommand(t, command, log=None):
     chan=t.open_session()
     exit_code=None
     chan.set_combine_stderr(True)
-    cmd_line=command+' ; echo exit_code=$?' # ESXi 3.5 don't return the exit_code   
+    cmd_line=command+' ; echo exit_code=$?'
     chan.exec_command(cmd_line)
     output=''
     while 1:
@@ -107,13 +107,11 @@ def RunCommand(t, command, log=None):
     exit_code=chan.recv_exit_status()
     chan.close()
     
-    if exit_code==-1:
-        # ESXi 3.5 don't return the exit_code   
-        for line in output.split('\n')[-2:]:
-            match=exit_code_re.match(line)
-            if match:
-                exit_code=int(match.group(1))
-                break
+    # ESX(i) don't return any exit_code
+    pos1=output.rfind('exit_code=')
+    pos2=output.find('\n', pos1)
+    exit_code=int(output[pos1+10:pos2])
+    output=output[:pos1]
 
     if log:
         l=log.debug
@@ -121,7 +119,7 @@ def RunCommand(t, command, log=None):
             l=log.warning
             
         if output or exit_code!=0:
-            l('code=%d command=%s', exit_code, cmd_line)
+            l('exit_code=%d command=%s', exit_code, cmd_line)
             for line in output.split('\n'):
                 l('> %s', line)
         
@@ -136,6 +134,8 @@ class Ghettovcb(Archiver):
     types=dict( backup='backup', copy='copy', move='move',)
     variables=dict(vm='vm_name')
 
+    initiate_backup_re=re.compile(r'.*info: Initiate backup for (?P<name>.+)')
+
     # -----------------------------------------------------------------------
     def __init__(self):
         Archiver.__init__(self)
@@ -152,17 +152,13 @@ class Ghettovcb(Archiver):
         self.ghettovcb_bin=None
         self.global_conf=None
         self.temp_target=None
-        self.current_vm=None
-        self.vm_var=None
         self.default_var=None
         
         self.destination=None
         self.type='backup'
-        self.target=''
         
         self.scp_bin=None
         self.scp_args=None
-
         
     # -----------------------------------------------------------------------
     def read_vm_conf(self, filename, default={}):
@@ -233,20 +229,21 @@ class Ghettovcb(Archiver):
         if job.get('target', None):
             self.target=job['target']
 
-
         self.global_conf=job.get('global_conf', None)
 
+        self.script=job.get('script', 'ghettoVCB.sh')
+
         if not os.path.isdir(self.local):
             errors['local']='not a directory'
         else:
             # search ghettoVCB.sh whatever the case
             self.ghettovcb=None
             list_dir=os.listdir(self.local)
-            if 'ghettoVCB.sh' in list_dir:
-                self.ghettovcb='ghettoVCB.sh'
+            if self.script in list_dir:
+                self.ghettovcb=self.script
             else:
                 for filename in os.listdir(self.local):
-                    if filename.lower()=='ghettovcb.sh':
+                    if filename.lower()==self.script.lower():
                         self.ghettovcb=filename
                         break
 
@@ -264,35 +261,25 @@ class Ghettovcb(Archiver):
                         self.global_conf=global_conf
 
             if not self.ghettovcb:
-                errors['local']='ghettoVCB.sh not found'
+                errors['local']=self.script+' not found'
             
             # read ghettoVCB.sh  and extract default variables
             self.default_var={}
             if self.ghettovcb:
                 self.default_var=self.read_vm_conf(os.path.join(self.local, self.ghettovcb))
                 
-                
+            # check ghettoVCB version
             if self.default_var.get('LAST_MODIFIED_DATE','')!=GHETTO_VERSION:
-                warnings['local']='ghettoVCB.sh version is different, use the one provided with MKBackup or be warned'
-            
+                warnings['local']=self.script+' version is different of MKBackup version, use the one provided with MKBackup or be warned'
             
             # read and extract gloabl_conf
             if self.global_conf:
                 self.default_var=self.read_vm_conf(os.path.join(self.local, self.global_conf), self.default_var)
             
-            # read vm config file if it exist
-            self.vm_var=dict()
-            for current_vm in self.vm_list:
-                filename=os.path.join(self.local, current_vm)
-                if os.path.isfile(filename):
-                    self.vm_var[current_vm]=self.read_vm_conf(filename, self.default_var)
-                else:
-                    self.vm_var[current_vm]=self.default_var
-            
         self.destination=job.get('destination', None)
         if self.destination:
             if not self.vm_list:
-                 warnings['destination']='vm_list must be set to use "destination"'
+                 errors['destination']='vm_list must be set to use "destination"'
                  
             self.destination=self.destination.replace('\n','')
             try:
@@ -327,7 +314,7 @@ class Ghettovcb(Archiver):
 
             self.temp_target=posixpath.join(self.remote_temp, os.path.basename(self.local))
             self.ghettovcb_bin=posixpath.join(self.temp_target, self.ghettovcb)
-            self.current_vm_file=posixpath.join(self.temp_target, 'current_vm')
+            self.include_vm_file=posixpath.join(self.temp_target, 'include_vm')
             self.exclude_vm_file=posixpath.join(self.temp_target, 'exclude_vm')
             
             _exit_code, _output=RunCommand(t, 'chmod +x "%s"' % self.ghettovcb_bin, log)
@@ -335,7 +322,12 @@ class Ghettovcb(Archiver):
             self.cmd_line='cd "%s" ; ./"%s" -d debug -c "%s"' % (self.temp_target, self.ghettovcb, self.temp_target)
             
             if self.vm_list:
-                self.cmd_line+=' -f "%s"' % (self.current_vm_file,)       
+                self.cmd_line+=' -f "%s"' % (self.include_vm_file,)       
+                for i, vm in enumerate(self.vm_list):
+                    if i==0:
+                        _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (vm, self.include_vm_file), log)
+                    else:
+                        _exit_code, _output=RunCommand(t, 'echo "%s" >> "%s"' % (vm, self.include_vm_file), log)
             else:
                 self.cmd_line+=' -a'       
                 if self.vm_exclude:
@@ -348,7 +340,7 @@ class Ghettovcb(Archiver):
             if self.global_conf:
                 self.cmd_line+=' -g "%s"' % (posixpath.join(self.temp_target, self.global_conf),)
             
-            self.cmd_line+=' ; echo exit_code=$?' # ESXi 3.5 don't return the exit_code   
+            self.cmd_line+=' ; echo exit_code=$?' # ESXi don't return the exit_code   
 
             extra+='cmd_line=%s\n' % self.cmd_line
             t.close()
@@ -365,32 +357,7 @@ class Ghettovcb(Archiver):
     # -----------------------------------------------------------------------
     def run(self, command, job, manager):
 
-        status=0
-        name=job['name']
-        
-        if self.vm_list:
-            vm_list=self.vm_list
-        else:
-            vm_list=[ None, ]
-                
-        for self.current_vm in vm_list:
-            if self.current_vm:
-                job['name']='%s.%s' % (name, self.current_vm)
-                
-            result=self.sub_run(command, job, manager)
-
-            code=send_mail_report(job, result, manager)
-            manager.start_logging()
-            status=max(status, code)
-
-        job['name']=name
-
-        return status
-        
-    # -----------------------------------------------------------------------
-    def sub_run(self, command, job, manager):
-
-        no_error=True
+        any_error=False
 
         log=manager.log
         now=manager.now
@@ -401,9 +368,6 @@ class Ghettovcb(Archiver):
         t.get_security_options()._set_ciphers(['blowfish-cbc', ])
         t.connect(username=self.login, password=self.password)
         # t.set_keepalive(300) # make the backup fail
-        if self.current_vm:
-            # create the current_vm_file
-            _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (self.current_vm, self.current_vm_file), log)
         chan=t.open_session()
         chan.set_combine_stderr(True)
         log.info('start: %s', self.cmd_line)
@@ -426,143 +390,185 @@ class Ghettovcb(Archiver):
         ghettovcb_exit_code=chan.recv_exit_status()
         chan.close()
         end=int(time.time())
+        log.debug('ghettovcb ssh exit code: %d', ghettovcb_exit_code)
         
-        for line in output.split('\n')[-2:]:
-            match=exit_code_re.match(line)
-            if match:
-                candidate=int(match.group(1))
-                if ghettovcb_exit_code==-1:
-                    ghettovcb_exit_code=candidate
-                elif ghettovcb_exit_code!=candidate:
-                    log.warning('ghettovcb exit_code mismatch ssh=%d log=%d', ghettovcb_exit_code, candidate)
-                    # use the one from SSH anyway
-                break
-                    
-        # search the target and make a directory listing 
-        dir_lst, up_lst, df, scp_process, target=None, None, None, None, None
-        if self.current_vm and self.vm_var[self.current_vm]['ENABLE_NON_PERSISTENT_NFS']!=1:
-            target=posixpath.join(self.vm_var[self.current_vm]['VM_BACKUP_VOLUME'], self.current_vm)
-            compress=self.vm_var[self.current_vm]['ENABLE_COMPRESSION']=='1'
-            if compress:
-                _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
-                dir_lst='== directory '+target+'\n'+dir_lst
+        # ESX(i) return an unreliable exit_code   
+        pos1=output.rfind('exit_code=')
+        pos2=output.find('\n', pos1)
+        ghettovcb_exit_code=int(output[pos1+10:pos2])
+        output=output[:pos1]
+
+        status=u''
+        attachements=[ # the type, subtype and coding is not used
+                       ('output.txt', None, output, 'text', 'plain', 'utf-8'),
+                    ]
+
+        # search in the output for the name of backed up vm
+        # should be == to self.vm_list when provided
+        vm_list=[]
+        for line in output.split('\n'):
+            current_error=False
+            match=self.initiate_backup_re.match(line)
+            if not match:
+                continue
+            
+            current_vm=match.group('name')
+            
+            # read vm config file if it exist
+            filename=os.path.join(self.local, current_vm)
+            if os.path.isfile(filename):
+                vm_var=self.read_vm_conf(filename, self.default_var)
             else:
-                _exit_code, up_lst=RunCommand(t, 'ls "%s"' % target, log)
-                dir_lst=None
-                # search the target directory, take the last one
-                target_name=''
-                for line in up_lst.split('\n'):
-                    if line.startswith(self.current_vm):
-                        if line>target_name:
-                           target_name=line
-                up_lst='== directory '+target+'\n'+up_lst
-                if target_name: 
-                    target=posixpath.join(target, target_name)
+                vm_var=self.default_var
+                    
+            # search the target and make a directory listing 
+            dir_lst, up_lst, scp_process, target=None, None, None, None
+            if vm_var['ENABLE_NON_PERSISTENT_NFS']!='1':
+                target=posixpath.join(vm_var['VM_BACKUP_VOLUME'], current_vm)
+                compress=vm_var['ENABLE_COMPRESSION']=='1'
+                if compress:
                     _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
                     dir_lst='== directory '+target+'\n'+dir_lst
-            # if destination, move or copy backup
-            if self.destination:
-                self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.current_vm))
-                if self.type in ('move', 'copy'):
-                    if not compress:
-                        # create the target dir, if don't exist
-                        try:
-                            if not os.path.isdir(self.target):
-                                os.makedirs(self.target)
-                        except Exception, e:
-                            log.error('creating directory %s: %s', self.target, e)
-                            no_error=False
-    
-                    if no_error:
-                        if not self.scp_bin:
-                            # use paramiko
-                            scpclient=MySCPClient(t, buff_size=65536)
-                            log.info('scp -r %s %s', target, self.target)
-                            scpclient.get(target, self.target, recursive=True)
-                        else:
-                            # use local SCP
-                            scp_args=self.scp_args[:]
-                            if not compress:
-                                if sys.platform in ('win32', ):
-                                    scp_args.append('-unsafe')
-                                #scp_args.append('-r')
-                                wildcard='/*'
+                else:
+                    _exit_code, up_lst=RunCommand(t, 'ls "%s"' % target, log)
+                    dir_lst=None
+                    # search the target directory, take the last one
+                    target_name=''
+                    for line in up_lst.split('\n'):
+                        if line.startswith(current_vm):
+                            if line>target_name:
+                               target_name=line
+                    up_lst='== directory '+target+'\n'+up_lst
+                    if target_name: 
+                        target=posixpath.join(target, target_name)
+                        _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
+                        dir_lst='== directory '+target+'\n'+dir_lst
+
+                if dir_lst:
+                    attachements.append(('dir-%s.txt' % (current_vm,), None, dir_lst, 'text', 'plain', 'utf-8'))
+        
+                if up_lst:
+                    attachements.append(('dirup-%s.txt' % (current_vm,), None, up_lst, 'text', 'plain', 'utf-8'))
+        
+                # if destination, move or copy backup
+                if self.destination:
+                    self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=current_vm))
+                    if self.type in ('move', 'copy'):
+                        if not compress:
+                            # create the target dir, if don't exist
+                            try:
+                                if not os.path.isdir(self.target):
+                                    os.makedirs(self.target)
+                            except Exception, e:
+                                log.error('creating directory %s: %s', self.target, e)
+                                current_error=True
+        
+                        if not current_error:
+                            start=int(time.time())
+                            status+='remote_target-%s=%s\r\n' % (current_vm, target)
+                            status+='target-%s=%s\r\n' % (current_vm, self.target)
+                            if not self.scp_bin:
+                                # use paramiko
+                                try:
+                                    scpclient=MySCPClient(t, buff_size=65536)
+                                    log.info('scp -r %s %s', target, self.target)
+                                    scpclient.get(target, self.target, recursive=True)
+                                except SCPException:
+                                    returncode=1
+                                    current_error=True
+                                else:
+                                    returncode=0
+                                status+='scp_exit_code-%s=%d\r\n' % (current_vm, returncode)
                             else:
-                                wildcard=''
+                                # use local SCP
+                                scp_args=self.scp_args[:]
+                                if not compress:
+                                    if sys.platform in ('win32', ):
+                                        scp_args.append('-unsafe')
+                                    #scp_args.append('-r')
+                                    wildcard='/*'
+                                else:
+                                    wildcard=''
+                                    
+                                scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
+                                scp_cmd=self.hide_password(' '.join(scp_args))
+                                log.info('scp_cmd=%s', scp_cmd)
+                                if sys.platform in ('win32', ):
+                                    scp_args=param_encode(scp_args, manager.default_encoding)
+                                scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                                scp_out, scp_err=scp_process.communicate()
+                                if scp_out:
+                                    attachements.append(('scp_out-%s.txt' % (current_vm,), None, scp_out, 'text', 'plain', manager.console_encoding))
+                                if scp_err:
+                                    attachements.append(('scp_err-%s.txt' % (current_vm,), None, scp_err, 'text', 'plain', manager.console_encoding))
+                                status+='scp_cmd-%s=%s\r\n' % (current_vm, scp_cmd)
+                                status+='scp_exit_code-%s=%d\r\n' % (current_vm, scp_process.returncode)
                                 
-                            scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
-                            scp_cmd=self.hide_password(' '.join(scp_args))
-                            log.info('scp_cmd=%s', scp_cmd)
-                            if sys.platform in ('win32', ):
-                                scp_args=param_encode(scp_args, manager.default_encoding)
-                            scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                            scp_out, scp_err=scp_process.communicate()
+                                if scp_process.returncode!=0:
+                                    current_error=True
+                                
+                            log.info('scp end')
                             
-                        log.info('scp end')
-                        
-                        if self.type=='move':
-                            # delete the source
-                            log.info('delete %s', target)
-                            _exit_code, _rm_out=RunCommand(t, 'rm -r "%s"' % target, log)
-    
-            # make the disk space usage
-            _exit_code, df=RunCommand(t, 'df', log)
+                            if self.type=='move':
+                                # delete the source
+                                log.info('delete %s', target)
+                                _exit_code, _rm_out=RunCommand(t, 'rm -r "%s"' % target, log)
+        
+                            try:
+                                total_free_bytes=free_space(self.target, log)
+                            except Exception, e:
+                                log.error('checking target directory: %s', e)
+                                status+='target_free_space-%s=%s\r\n' % (current_vm, e)
+                            else:
+                                status+='target_free_space-%s=%d\r\n' % (current_vm, total_free_bytes)
+                
+                            dir_out, _total_size=list_dir(self.target, log)        
+                            attachements.append(('target_dir-%s.txt' % (current_vm,), None, dir_out.encode('utf-8'), 'text', 'plain', 'utf-8'))
+
+                            end=int(time.time())
+                            status+='scp_start-%s=%s\r\n' % (current_vm, time.ctime(start))
+                            status+='scp_end-%s=%s\r\n' % (current_vm, time.ctime(end))
+                            status+='scp_start_epoch-%s=%d\r\n' % (current_vm, start)
+                            status+='scp_end_epoch-%s=%d\r\n' % (current_vm, end)
+
+
+            any_error=any_error or current_error
+        
+        # make the disk space usage
+        _exit_code, df=RunCommand(t, 'df', log)
+        
+        # make volume listing 
+        _exit_code, ls_volumes=RunCommand(t, 'ls -l /vmfs/volumes', log)
+        ls_volumes='== directory /vmfs/volumes\n'+ls_volumes
 
         t.close()
 
-        if ghettovcb_exit_code==0 and (not scp_process or scp_process.returncode==0) and no_error:
+        if ghettovcb_exit_code==0 and not any_error:
             backup_status='OK'
         else:
             backup_status='ERR'
 
-        status=u''
-        status+='name=%s\r\n' % job['name']
-        status+='program=%s\r\n' % self.name
-        status+='version=%s\r\n' % self.__version__
-        status+='status=%s\r\n' % backup_status
-        status+='hostname=%s\r\n' % manager.hostname
+        status_pre=u''
+        status_pre+='name=%s\r\n' % job['name']
+        status_pre+='program=%s\r\n' % self.name
+        status_pre+='version=%s\r\n' % self.__version__
+        status_pre+='status=%s\r\n' % backup_status
+        status_pre+='hostname=%s\r\n' % manager.hostname
         
-        status+='exit_code=%d\r\n' % ghettovcb_exit_code
-        status+='start=%s\r\n' % time.ctime(start)
-        status+='end=%s\r\n' % time.ctime(end)
-        status+='start_epoch=%d\r\n' % start
-        status+='end_epoch=%d\r\n' % end
-
-        status+='type=%s\r\n' % self.type
-        status+='target=%s\r\n' % self.target
-            
-
-        attachements=[ # the type, subtype and coding is not used
-                       ('output.txt', None, output, 'text', 'plain', 'utf-8'),
-                    ]
-        if dir_lst:
-            attachements.append(('dir.txt', None, dir_lst, 'text', 'plain', 'utf-8'))
-
-        if up_lst:
-            attachements.append(('dir_up.txt', None, up_lst, 'text', 'plain', 'utf-8'))
+        status_pre+='exit_code=%d\r\n' % ghettovcb_exit_code
+        status_pre+='start=%s\r\n' % time.ctime(start)
+        status_pre+='end=%s\r\n' % time.ctime(end)
+        status_pre+='start_epoch=%d\r\n' % start
+        status_pre+='end_epoch=%d\r\n' % end
 
+        status_pre+='type=%s\r\n' % self.type
+        status=status_pre+status
+        
         if df:
-            attachements.append(('df.txt', None, df, 'text', 'plain', 'utf-8'),)
+            attachements.insert(0, ('df.txt', None, df, 'text', 'plain', 'utf-8'),)
 
-        if scp_process:
-            status+='scp_cmd=%s\r\n' % scp_cmd
-            status+='scp_exit_code=%d\r\n' % scp_process.returncode
-            status+='remote_target=%s\r\n' % target
-            if scp_out:
-                attachements.append(('scp_out.txt', None, scp_out, 'text', 'plain', manager.console_encoding))
-            if scp_err:
-                attachements.append(('scp_err.txt', None, scp_err, 'text', 'plain', manager.console_encoding))
-
-            try:
-                total_free_bytes=free_space(self.target, log)
-            except Exception, e:
-                log.error('checking target directory: %s', e)
-                status+='target_free_space=%s\r\n' % e
-            else:
-                status+='target_free_space=%d\r\n' % total_free_bytes
-
-            dir_out, _total_size=list_dir(self.target, log)        
-            attachements.append(('target_dir.txt', None, dir_out.encode('utf-8'), 'text', 'plain', 'utf-8'))
+        if ls_volumes:
+            attachements.insert(0, ('ls_volumes.txt', None, ls_volumes, 'text', 'plain', 'utf-8'),)
 
         
         return backup_status, status, attachements
