@@ -8,8 +8,9 @@
 #
 # Supported archiver:
 # - ntbackup
-# - wbadmin (next to come)
-# - tar (next to come)
+# - wbadmin
+# - tar
+# - ghettoVCB
 
 # Features:
 # - job definitions are stored in a configuration file
@@ -34,7 +35,7 @@ from archiver import boolean, check_mail_config, Destinations, sendmail, Manager
 
 import cron
 
-__version__='0.8.7b'
+__version__='0.9.2'
 last_version_url='http://www.magikmon.com/download/mkbackup_last.txt'
 
 
@@ -42,7 +43,7 @@ def gen_archiver(name):
     name=name.lower()
     if name=='ntbackup':
         import ntbackup
-        return ntbackup.NtBackup
+        return ntbackup.NTBackup
     elif name=='tar':
         import tar
         return tar.Tar
@@ -86,35 +87,38 @@ def update_check():
 
 
 # ---------------------------------------------------------------------------        
-def mail_exception(start, job, extra, manager):
+def mail_exception(start, job, extra, manager, subject):
     
     end=int(time.time())
-    subject='MKBACKUP BACKUP ERR %s' % (job['name'], )
-    msg_body=job['msg_header']
-    msg_body+='\nUnexpected error, read "traceback.txt" for more\n\n'
-    msg_body+=extra+'\n'
-    tb=StringIO.StringIO()
-    traceback.print_exc(None, tb)
-    
+
     status=u''
     status+='name=%s\r\n' % job['name']
     status+='program=%s\r\n' % job.get('program','')
     status+='version=%s\r\n' % __version__
     status+='status=ERR\r\n'
     status+='hostname=%s\r\n' % platform.node()
-    status+='runtime_error=%s\r\n' % sys.exc_info()[1]
     status+='start=%s\r\n' % time.ctime(start)
     status+='end=%s\r\n' % time.ctime(end)
     status+='start_epoch=%d\r\n' % start
     status+='end_epoch=%d\r\n' % end
+    status+='runtime_error=%r\r\n' % sys.exc_info()[1]
+    
+    # TODEL: subject='MKBACKUP BACKUP ERR %s' % (job['name'], )
+    msg_body=job['msg_header']
+    msg_body+='\nUnexpected error, read "traceback.txt" for more\n\n'
+    msg_body+='\n%s\n' % status
+    msg_body+=extra+'\n'
 
+    tb=StringIO.StringIO()
+    traceback.print_exc(None, tb)
+    
     attachements=manager.attachements[:]
     attachements+=[ # the type, subtype and coding is not used
            ('traceback.txt', None, tb.getvalue().encode('utf-8'), 'text', 'plain', 'utf-8'),
            ('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'),
     ]
     tb.close()
-    sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'])
+    errmsg=sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'], job['attachment_size'], manager.log)
 
 
 class MyFormatter(logging.Formatter):
@@ -165,7 +169,10 @@ else:
 
 if cmd_options.verbose:
     console=logging.StreamHandler()
-    console.setFormatter(MyFormatter('%(asctime)s,%(msecs)03d %(levelname)-3.3s %(message)s', '%H:%M:%S', encoding=sys.stderr.encoding))
+    encoding=sys.stderr.encoding
+    if encoding==None:
+        encoding='ascii'
+    console.setFormatter(MyFormatter('%(asctime)s,%(msecs)03d %(levelname)-3.3s %(message)s', '%H:%M:%S', encoding=encoding))
     log.addHandler(console)
 
 
@@ -194,6 +201,7 @@ config_default=dict(verify='no',
                     smtp_port='25',
                     smtp_host='127.0.0.1',
                     night_shift='yes',
+                    attachment_size='100',
                     )
 
 try:
@@ -333,7 +341,8 @@ for job_name in job_list:
     except Exception, e:
         log.exception('loading job %s by program %s', job_name, program)
         if job['mail'] and not mail_config_errors:
-            mail_exception(start, job, '', manager)
+            subject='MKBACKUP CONFIG ERR %s ' % (job['name'], )
+            mail_exception(start, job, '', manager, subject)
         continue
 
     for line in extra.split('\n'):
@@ -346,19 +355,28 @@ for job_name in job_list:
         
     if errors:
         exit_code=3
-        msg_body+='Error in section: %s\n\n' % job['name']
-        log.error('Error in section: %s', job['name'])
+        msg_body+='Errors in section: %s\n\n' % job['name']
+        log.error('Errors in section: %s', job['name'])
         for k, v in errors.iteritems():
             log.error('%s: %s', k, v)
             msg_body+='%s\n    %s\n\n' % (k, v)
+
+        for k, v in warnings.iteritems():
+            log.warning('%s: %s', k, v)
+            msg_body+='%s\n    %s\n\n' % (k, v)
                 
         if job['mail'] and not mail_config_errors and not command=='check':
             subject='MKBACKUP CONFIG ERR %s ' % (job['name'], )
-            sendmail(job['sender'], job['recipients'], subject, msg_body, manager.attachements, job['smtp_host'], job['smtp_port'])
+            errmsg=sendmail(job['sender'], job['recipients'], subject, msg_body, manager.attachements, job['smtp_host'], job['smtp_port'], job['attachment_size'], manager.log)
 
         continue
 
-    log.info('No error in section: %s', job['name'])
+    if warnings:
+        log.warning('Warnings in section: %s', job['name'])
+        for k, v in warnings.iteritems():
+            log.warning('%s: %s', k, v)
+    else:
+        log.info('No errors in section: %s', job['name'])
 
     destination=getattr(arch, 'destination', None)
     
@@ -387,7 +405,7 @@ for job_name in job_list:
         msg_body='%s\n\n%s\n' % (msg_body, extra)
         if job['mail'] and command=='checkmail':
             subject='MKBACKUP CONFIG OK %s ' % (job['name'], )
-            sendmail(job['sender'], job['recipients'], subject, msg_body, manager.attachements, job['smtp_host'], job['smtp_port'])
+            errmsg=sendmail(job['sender'], job['recipients'], subject, msg_body, manager.attachements, job['smtp_host'], job['smtp_port'], job['attachment_size'], manager.log)
         continue 
 
     #print 'destination', destination
@@ -404,16 +422,21 @@ for job_name in job_list:
         exit_code=2
         log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
         if job['mail']:
-            mail_exception(start, job, extra, manager)
+            subject='MKBACKUP BACKUP ERR %s ' % (job['name'], )
+            mail_exception(start, job, extra, manager, subject)
     else:
         log.info('end command=%s job=%s archiver=%s', command, job_name, program)
 
         if isinstance(result, int):
             exit_code=max(exit_code, result)
         else:
-            code=send_mail_report(job, result, manager)
-            exit_code=max(exit_code, code)
-            
+            if result[1]!='OK':
+                exit_code=max(exit_code, 1)
+
+            if not job['mail']:
+                log.info('mail is disabled')
+            else:
+                errmsg=send_mail_report(job, result, manager)
 
 sys.exit(exit_code)
 
